#!/usr/bin/env node
const CLUSTER_URL = 'https://api.coded.sh';

const exec = require('child_process').exec;
const inquirer = require('inquirer');
const Parse = require('parse/node');
const figlet = require('figlet');
const request = require('request');
const version = 'ver 1.0\n';

figlet('CODED', {
    font: 'Small Slant',
    horizontalLayout: 'full',
    verticalLayout: 'default'
}, function (err, data) {
    if (err == null) {
        console.log(data);
    }
    console.log(version);
    start();
});

const parseAppId = 'coded';
Parse.initialize(parseAppId);
Parse.serverURL = `${CLUSTER_URL}/parse`;
Parse.User.enableUnsafeCurrentUser();

const questions = [
    {
        type: 'input',
        name: 'email',
        message: 'email'
  },
    {
        type: 'password',
        name: 'password',
        message: 'password'
  }
];

var choices = [{
    name: 'Login'
    }, {
    name: 'Register'
    }];

function start() {

    var message = 'Hey. I will help to install Coded Node on your server. Log in or create a new account to start installation.';
    inquirer.prompt([
        {
            type: 'list',
            message: message,
            name: 'actions',
            choices: choices,
            validate: function (answer) {
                return true;
            }
    }
  ]).then(answers => {
        if (answers.actions == 'Login') {
            checkUserLogin();
        } else if (answers.actions == 'Register') {
            checkUserRegistration();
        }
    });

}

function checkUserLogin() {
    inquirer.prompt(questions).then(answers => {
        var user = Parse.User.logIn(answers.email, answers.password, {
            success: function (user) {

                Parse.User.become(user.getSessionToken()).then(function (user) {
                    console.log('Logged as ' + user.get("username"));
                    getNetworkData(user.getSessionToken());
                }, function (error) {

                });

            },
            error: function (user, error) {
                // Show the error message somewhere and let the user try again.
                console.log("Error: " + error.code + " " + error.message);
            }
        });

    });
}

function checkUserRegistration() {
    inquirer.prompt(questions).then(answers => {
        signUpUser(answers);
    });
}

function signUpUser(answers) {
    var user = new Parse.User();
    user.set("username", answers.email);
    user.set("email", answers.email);
    user.set("password", answers.password);
    user.signUp().then((user) => {
            Parse.User.become(user.getSessionToken()).then(function (user) {
                console.log('Logged as ' + user.get("username"));
                getNetworkData(user.getSessionToken());
            }, function (error) {
                console.log('Ooops. Cannot register. ' + error);
            });
        },
        (error) => {
            spinner.stop(true);
            console.log(error.message);
        });
}

function getNetworkData(token) {
    //Get interface
    var network = require('network');
    network.get_public_ip(function (err, ip) {
        //console.log(err || ip); // should return your public IP address
        if (err == null) {
            network.get_active_interface(function (err, obj) {
                //console.log(err || obj); // should return your public IP address
                if (err == null) {
                    var publicIp = ip;
                    var ethInterface = obj.name;
                    installCoded(ethInterface, token, publicIp);
                }
            });
        }
    });
}

function installCoded(ethInterface, token, publicIp) {
  var appsOptions = {
      method: 'POST',
      json: true,
      url: `${CLUSTER_URL}/node`,
      body: {
        "serverIP":publicIp,
        "token":token,
        "status":"install"
      },
      headers: {
          "Content-Type": "application/json"
      }
  };
  request(appsOptions, function (error, response, body) {
    if (error != null){
      console.error(error);
      return;
    }
    var serverName = body.objectId.toLowerCase();
    var command = `curl ${CLUSTER_URL}/node-config/initial-server-setup/` + serverName + '/' + ethInterface + '/' + publicIp + ` -o coded-install.sh && chmod u+x coded-install.sh && ./coded-install.sh`;
    var coffeeProcess = exec(command, {
        maxBuffer: 1024 * 2048
    });
    coffeeProcess.on('close', function (code) {
        console.log('* * * * * * * * * * * * * * * * * *');
        var text = require('figlet');
        text('SERVER READY', {
            font: 'Small Slant',
            horizontalLayout: 'full',
            verticalLayout: 'default'
        }, function (err, data) {
            if (err != null) {
              console.error(err);
                return;
            }
            var updateServreOptions = {
                method: 'PUT',
                json: true,
                url: `${CLUSTER_URL}/node`,
                body: {
                  "serverIP":publicIp,
                  "token":token,
                  "status":"ready"
                },
                headers: {
                    "Content-Type": "application/json"
                }
            };
            request(updateServreOptions, function (error, response, body) {
              if (error != null){
                console.error(error);
                return;
              }
              console.log(data);
              console.log(`\nYour workplace is ready. Install Coded CLI (coded.sh/docs/#/gettingstarted) on you local system to deploy a first project.\n`);
              console.log('* * * * * * * * * * * * * * * * * *');
              exec('reboot', function (err, stdout, stderr) {});
            });
        });
    });
    coffeeProcess.stdout.pipe(process.stdout);
    coffeeProcess.stderr.pipe(process.stderr);
  });
}
